<?php
/*
 * VDev.php
 *
 * part of pfSense (https://www.pfsense.org)
 * Copyright (c) 2013-2022 Rubicon Communications, LLC (Netgate)
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

namespace Netgate\Services\ZFS;

use Nette\Utils\Arrays;
use Nette\Utils\Strings;

final class VDev implements ZFSNode {
	private $parent = null;

	private $vdev = [];

	private $child_vdevs = [];

	function __construct($parent, $vdev) {
		$this->parent = $parent;

		$this->vdev = $vdev;

		$this->_initChildVdevs();
	}

	public function getChildVdevsAndSelf() {
		$vdevs = [$this,];

		foreach ($this->getChildVdevs() as $child) {
			$children = $child->getChildVdevsAndSelf();

			$vdevs = array_merge($vdevs, $children);
		}

		return $vdevs;
	}

	public function getTypeFriendly() {
		$prefer_properties = array('path', 'type', 'guid');

		foreach ($prefer_properties as $property) {
			$property = $this->getProperty($property);

			if (!empty($property)) {
				return $property;
			}
		}

		return null;
	}

	public function getProperty($key, $default = null) {
		return Arrays::get($this->vdev, $key, $default);
	}

	public function getGuid() {
		return $this->getProperty('guid');
	}

	public function getName() {
		return $this->getProperty('name');
	}

	public function getParent() : ZFSNode {
		return $this->parent;
	}

	public function getHtmlClass(string $prefix = null) : string {
		return Strings::webalize("{$this->getParent()->getHtmlClass($prefix)}-{$this->getGuid()}");
	}

	public function getParentHtmlClass(string $prefix = null) : string {
		return Strings::webalize($this->getParent()->getHtmlClass($prefix));
	}

	public function hasChildVdevs() {
		return !empty($this->_getChildren());
	}

	public function getChildVdevs() {
		if (empty($this->child_vdevs)) {
			$this->_initChildVdevs();
		}

		return $this->child_vdevs;
	}

	private function _getChildren() {
		return $this->getProperty('children');
	}

	private function _initChildVdevs() {
		if ($this->hasChildVdevs()) {
			$this->child_vdevs = array_map(function($vdev) {
				return $this->_getNewVdevObject($this, $vdev);
			}, $this->_getChildren());
		}
	}

	private function _getNewVdevObject($parent, $vdev) {
		return (new VDev($parent, $vdev));
	}

}